@@ -11,8 +11,7 @@
  * information regarding copyright ownership.
  */
 
-#ifndef NS_CLIENT_H
-#define NS_CLIENT_H 1
+#pragma once
 
 /*****
 ***** Module Info
@@ -63,7 +62,6 @@
 #include <isc/buffer.h>
 #include <isc/magic.h>
 #include <isc/netmgr.h>
-#include <isc/platform.h>
 #include <isc/quota.h>
 #include <isc/stdtime.h>
 
@@ -73,7 +71,6 @@
 #include <dns/name.h>
 #include <dns/rdataclass.h>
 #include <dns/rdatatype.h>
-#include <dns/tcpmsg.h>
 #include <dns/types.h>
 
 #include <ns/query.h>
@@ -150,31 +147,24 @@ struct ns_clientmgr {
 	ns_server_t    *sctx;
 	isc_taskmgr_t  *taskmgr;
 	isc_timermgr_t *timermgr;
-	isc_task_t     *excl;
 	isc_refcount_t	references;
-	int		ncpus;
+	int		tid;
 
 	/* Attached by clients, needed for e.g. recursion */
-	isc_task_t **taskpool;
+	isc_task_t *task;
 
-	ns_interface_t *interface;
-
-	/* Lock covers manager state. */
-	isc_mutex_t lock;
-	bool	    exiting;
+	dns_aclenv_t *aclenv;
 
 	/* Lock covers the recursing list */
 	isc_mutex_t   reclock;
 	client_list_t recursing; /*%< Recursing clients */
-
-	/*%< mctx pool for clients. */
-	isc_mem_t **mctxpool;
 };
 
 /*% nameserver client structure */
 struct ns_client {
 	unsigned int	 magic;
 	isc_mem_t	*mctx;
+	int		 tid;
 	bool		 allocated; /* Do we need to free it? */
 	ns_server_t	*sctx;
 	ns_clientmgr_t	*manager;
@@ -197,9 +187,11 @@ struct ns_client {
 	dns_message_t  *message;
 	unsigned char  *sendbuf;
 	dns_rdataset_t *opt;
+	dns_ednsopt_t  *ede;
 	uint16_t	udpsize;
 	uint16_t	extflags;
 	int16_t		ednsversion; /* -1 noedns */
+	uint16_t	additionaldepth;
 	void (*cleanup)(ns_client_t *);
 	ns_query_t    query;
 	isc_time_t    requesttime;
@@ -218,7 +210,6 @@ struct ns_client {
 	dns_ecs_t ecs; /*%< EDNS client subnet sent by client */
 
 	struct in6_pktinfo pktinfo;
-	isc_dscp_t	   dscp;
 	/*%
 	 * Information about recent FORMERR response(s), for
 	 * FORMERR loop avoidance.  This is separate for each
@@ -278,11 +269,7 @@ struct ns_client {
  */
 #define NS_FAILCACHE_CD 0x01
 
-#if defined(_WIN32) && !defined(_WIN64)
-LIBNS_EXTERNAL_DATA extern atomic_uint_fast32_t ns_client_requests;
-#else  /* if defined(_WIN32) && !defined(_WIN64) */
-LIBNS_EXTERNAL_DATA extern atomic_uint_fast64_t ns_client_requests;
-#endif /* if defined(_WIN32) && !defined(_WIN64) */
+extern atomic_uint_fast64_t ns_client_requests;
 
 /***
  *** Functions
@@ -318,6 +305,12 @@ ns_client_error(ns_client_t *client, isc_result_t result);
  * will have an RCODE determined by 'result'.
  */
 
+void
+ns_client_extendederror(ns_client_t *client, uint16_t code, const char *text);
+/*%<
+ * Set extended error with INFO-CODE <code> and EXTRA-TEXT <text>.
+ */
+
 void
 ns_client_drop(ns_client_t *client, isc_result_t result);
 /*%<
@@ -346,18 +339,24 @@ ns_client_settimeout(ns_client_t *client, unsigned int seconds);
  */
 
 isc_result_t
-ns_clientmgr_create(isc_mem_t *mctx, ns_server_t *sctx, isc_taskmgr_t *taskmgr,
-		    isc_timermgr_t *timermgr, ns_interface_t *ifp, int ncpus,
+ns_clientmgr_create(ns_server_t *sctx, isc_taskmgr_t *taskmgr,
+		    isc_timermgr_t *timermgr, dns_aclenv_t *aclenv, int tid,
 		    ns_clientmgr_t **managerp);
 /*%<
  * Create a client manager.
  */
 
 void
-ns_clientmgr_destroy(ns_clientmgr_t **managerp);
+ns_clientmgr_shutdown(ns_clientmgr_t *manager);
 /*%<
- * Destroy a client manager and all ns_client_t objects
- * managed by it.
+ * Shutdown a client manager and all ns_client_t objects
+ * managed by it
+ */
+
+void
+ns_clientmgr_detach(ns_clientmgr_t **managerp);
+/*%<
+ * Detach from a client manager.
  */
 
 isc_sockaddr_t *
@@ -575,5 +574,3 @@ ns__client_put_cb(void *client0);
  * Free all resources allocated to this client object, so that
  * it can be freed.
  */
-
-#endif /* NS_CLIENT_H */
